@@ -1,18 +1,28 @@
+use crate::{hashing::djb2, log_e};
+use serde::{Deserialize, Deserializer, Serialize, Serializer};
+use serde_json::{
+    value::{to_raw_value, RawValue},
+    Value as JsonValue,
+};
 use std::{
     collections::HashMap,
     sync::{Arc, Mutex, Weak},
 };
 
-use serde::{Deserialize, Deserializer, Serialize, Serializer};
-use serde_json::{value::RawValue, Value as JsonValue};
-
-use crate::{hashing::djb2, log_e};
-
 const TAG: &str = "DynamicReturnable";
 
 lazy_static::lazy_static! {
-    static ref MEMOIZED_VALUES: Mutex<HashMap<String, Weak<MemoizedValue>>> =
+    pub(crate) static ref MEMOIZED_VALUES: Mutex<HashMap<String, Weak<MemoizedValue>>> =
         Mutex::new(HashMap::new());
+
+    static ref EMPTY_DYNAMIC_RETURNABLE: DynamicReturnable = DynamicReturnable {
+        hash: "".to_string(),
+        value: Arc::new(MemoizedValue {
+            raw_value: None,
+            bool_value: None,
+            json_value: None,
+        }),
+    };
 }
 
 #[derive(Clone, PartialEq, Debug)]
@@ -22,6 +32,29 @@ pub struct DynamicReturnable {
 }
 
 impl DynamicReturnable {
+    pub fn empty() -> Self {
+        EMPTY_DYNAMIC_RETURNABLE.clone()
+    }
+
+    pub fn from_map(value: HashMap<String, JsonValue>) -> Self {
+        let raw_value = match to_raw_value(&value) {
+            Ok(raw_value) => raw_value,
+            Err(e) => {
+                log_e!(TAG, "Failed to convert map to raw value: {}", e);
+                return Self::empty();
+            }
+        };
+
+        let hash = djb2(raw_value.get());
+        let value = Arc::new(MemoizedValue {
+            raw_value: Some(raw_value),
+            bool_value: None,
+            json_value: Some(value.clone()),
+        });
+
+        Self::new(hash.to_string(), value)
+    }
+
     pub fn get_bool(&self) -> Option<bool> {
         self.value.bool_value
     }
@@ -30,7 +63,14 @@ impl DynamicReturnable {
         self.value.json_value.clone()
     }
 
+    pub fn get_json_ref(&self) -> Option<&HashMap<String, JsonValue>> {
+        self.value.json_value.as_ref()
+    }
+
     fn new(hash: String, value: Arc<MemoizedValue>) -> Self {
+        let weak_value = Arc::downgrade(&value);
+        set_memoized_value(&hash, weak_value);
+
         Self { hash, value }
     }
 }
@@ -40,19 +80,19 @@ impl<'de> Deserialize<'de> for DynamicReturnable {
     where
         D: Deserializer<'de>,
     {
-        let raw_value: &'de RawValue = Deserialize::deserialize(deserializer)?;
-        let raw_value_str = raw_value.get();
+        let raw_value_ref: &'de RawValue = Deserialize::deserialize(deserializer)?;
+
+        let raw_value_str = raw_value_ref.get();
         let hash = djb2(raw_value_str);
 
         if let Some(value) = get_memoized_value(&hash) {
             return Ok(DynamicReturnable { hash, value });
         }
 
-        let value = MemoizedValue::new(raw_value_str);
-        let weak_value = Arc::downgrade(&value);
+        let raw_value = raw_value_ref.to_owned();
+        let value = MemoizedValue::new(raw_value);
 
         let new_returnable = DynamicReturnable::new(hash.clone(), value);
-        set_memoized_value(&hash, weak_value);
 
         Ok(new_returnable)
     }
@@ -67,6 +107,10 @@ impl Serialize for DynamicReturnable {
             return bool_value.serialize(serializer);
         }
 
+        if let Some(raw_value) = &self.value.raw_value {
+            return raw_value.serialize(serializer);
+        }
+
         if let Some(json_value) = &self.value.json_value {
             return json_value.serialize(serializer);
         }
@@ -96,17 +140,27 @@ impl Drop for DynamicReturnable {
     }
 }
 
-#[derive(Debug, Clone, PartialEq)]
-struct MemoizedValue {
-    bool_value: Option<bool>,
-    json_value: Option<HashMap<String, JsonValue>>,
+#[derive(Debug, Clone)]
+pub(crate) struct MemoizedValue {
+    pub(crate) raw_value: Option<Box<RawValue>>,
+    pub(crate) bool_value: Option<bool>,
+    pub(crate) json_value: Option<HashMap<String, JsonValue>>,
+}
+
+impl PartialEq for MemoizedValue {
+    fn eq(&self, other: &Self) -> bool {
+        self.raw_value.as_ref().map(|v| v.get()) == other.raw_value.as_ref().map(|v| v.get())
+            && self.bool_value == other.bool_value
+            && self.json_value == other.json_value
+    }
 }
 
 impl MemoizedValue {
-    fn new(raw_json: &str) -> Arc<Self> {
-        let value = match raw_json {
-            "true" | "false" => Self::from_bool(raw_json == "true"),
-            raw_json => Self::from_object_str(raw_json),
+    fn new(raw_value: Box<RawValue>) -> Arc<Self> {
+        let value = match raw_value.get() {
+            "true" => Self::from_bool(true),
+            "false" => Self::from_bool(false),
+            _ => Self::from_raw_value(raw_value),
         };
 
         Arc::new(value)
@@ -114,13 +168,14 @@ impl MemoizedValue {
 
     fn from_bool(bool_value: bool) -> Self {
         Self {
+            raw_value: None,
             bool_value: Some(bool_value),
             json_value: None,
         }
     }
 
-    fn from_object_str(raw_json: &str) -> Self {
-        let json_value = match serde_json::from_str(raw_json) {
+    fn from_raw_value(raw_value: Box<RawValue>) -> Self {
+        let json_value = match serde_json::from_str(raw_value.get()) {
             Ok(json_value) => json_value,
             Err(e) => {
                 log_e!(TAG, "Failed to parse json: {}", e);
@@ -129,6 +184,7 @@ impl MemoizedValue {
         };
 
         Self {
+            raw_value: Some(raw_value),
             bool_value: None,
             json_value,
         }
