@@ -16,7 +16,6 @@ pub use override_adapter::{
     statsig_local_override_adapter::StatsigLocalOverrideAdapter,
 };
 pub use persistent_storage::persistent_storage_trait::*;
-pub use sampling_processor::SamplingProcessor;
 pub use spec_store::SpecStore;
 pub use specs_adapter::*;
 pub use statsig::FailureDetails;
@@ -29,11 +28,13 @@ pub use statsig_core_api_options::{
 pub use statsig_err::StatsigErr;
 pub use statsig_options::StatsigOptions;
 pub use statsig_runtime::StatsigRuntime;
+pub use user::user_data::UserData as StatsigUserData;
 pub use user::{StatsigUser, StatsigUserBuilder};
 
 pub mod compression;
 pub mod data_store_interface;
 pub mod evaluation;
+pub mod event_logging;
 pub mod global_configs;
 pub mod hashing;
 pub mod hashset_with_ttl;
@@ -50,7 +51,6 @@ pub mod statsig_types;
 pub mod user;
 
 mod dcs_str;
-mod event_logging;
 mod event_logging_adapter;
 mod gcir;
 mod id_lists_adapter;
@@ -59,7 +59,6 @@ mod initialize_response;
 mod macros;
 mod observability;
 mod persistent_storage;
-mod sampling_processor;
 mod spec_store;
 mod specs_adapter;
 mod statsig;
