@@ -9,7 +9,7 @@ use crate::evaluation::evaluator_result::{
     result_to_dynamic_config_eval, result_to_experiment_eval, result_to_gate_eval,
     result_to_layer_eval, EvaluatorResult,
 };
-use crate::evaluation::ua_parser::UserAgentParser;
+use crate::evaluation::user_agent_parsing::UserAgentParser;
 use crate::event_logging::event_logger::{EventLogger, ExposureTrigger};
 use crate::event_logging::event_queue::queued_config_expo::EnqueueConfigExpoOp;
 use crate::event_logging::event_queue::queued_experiment_expo::EnqueueExperimentExpoOp;
@@ -55,6 +55,7 @@ use crate::{
         LayerEvaluationOptions, ParameterStoreEvaluationOptions,
     },
 };
+use chrono::Utc;
 use serde::de::DeserializeOwned;
 use serde::Serialize;
 use serde_json::json;
@@ -83,9 +84,9 @@ pub struct Statsig {
 
     sdk_key: String,
     event_logger: Arc<EventLogger>,
-    specs_adapter: Arc<dyn SpecsAdapter>,
+    specs_adapter: SpecsAdapterHousing,
     event_logging_adapter: Arc<dyn EventLoggingAdapter>,
-    id_lists_adapter: Option<Arc<dyn IdListsAdapter>>,
+    id_lists_adapter: IdListsAdapterHousing,
     override_adapter: Option<Arc<dyn OverrideAdapter>>,
     spec_store: Arc<SpecStore>,
     hashing: Arc<HashUtil>,
@@ -157,6 +158,24 @@ impl InitializeDetails {
     }
 }
 
+impl Drop for Statsig {
+    fn drop(&mut self) {
+        self.event_logger.force_shutdown();
+
+        if let Some(adapter) = &self.id_lists_adapter.as_default_adapter {
+            adapter.force_shutdown();
+        }
+
+        if let Some(adapter) = &self.specs_adapter.as_default_adapter {
+            adapter.force_shutdown();
+        }
+
+        shutdown_output_logger();
+
+        log_d!(TAG, "Statsig instance dropped");
+    }
+}
+
 impl Statsig {
     pub fn new(sdk_key: &str, options: Option<Arc<StatsigOptions>>) -> Self {
         let statsig_runtime = StatsigRuntime::get_runtime();
@@ -332,7 +351,7 @@ impl Statsig {
                 ))
             }
             sub_result = async {
-                let id_list_shutdown: Pin<Box<_>> = if let Some(adapter) = &self.id_lists_adapter {
+                let id_list_shutdown: Pin<Box<_>> = if let Some(adapter) = &self.id_lists_adapter.inner {
                     adapter.shutdown(timeout)
                 } else {
                     Box::pin(async { Ok(()) })
@@ -343,7 +362,7 @@ impl Statsig {
                 try_join!(
                     id_list_shutdown,
                     self.event_logger.shutdown(),
-                    self.specs_adapter.shutdown(timeout, &self.statsig_runtime),
+                    self.specs_adapter.inner.shutdown(timeout, &self.statsig_runtime),
                 )
             } => {
                 match sub_result {
@@ -397,8 +416,7 @@ impl Statsig {
                 ops_stats,
                 TAG,
                 StatsigErr::SpecsAdapterSkipPoll(format!(
-                    "Failed to schedule specs adapter background job: {}",
-                    e
+                    "Failed to schedule specs adapter background job: {e}"
                 ))
             );
         }
@@ -418,8 +436,8 @@ impl Statsig {
         let timeout_future = sleep(timeout);
 
         let statsig_runtime = self.statsig_runtime.clone();
-        let id_lists_adapter = self.id_lists_adapter.clone();
-        let specs_adapter = self.specs_adapter.clone();
+        let id_lists_adapter = self.id_lists_adapter.inner.clone();
+        let specs_adapter = self.specs_adapter.inner.clone();
         let ops_stats = self.ops_stats.clone();
         let background_tasks_started = self.background_tasks_started.clone();
         // Create another clone specifically for the closure
@@ -450,7 +468,7 @@ impl Statsig {
     async fn initialize_impl_with_details(&self) -> Result<InitializeDetails, StatsigErr> {
         let start_time = Instant::now();
         self.spec_store.set_source(SpecsSource::Loading);
-        self.specs_adapter.initialize(self.spec_store.clone());
+        self.specs_adapter.inner.initialize(self.spec_store.clone());
 
         let mut error_message = None;
         let mut id_list_ready = None;
@@ -473,6 +491,7 @@ impl Statsig {
 
         let init_res = match self
             .specs_adapter
+            .inner
             .clone()
             .start(&self.statsig_runtime)
             .await
@@ -485,7 +504,7 @@ impl Statsig {
             }
         };
 
-        if let Some(adapter) = &self.id_lists_adapter {
+        if let Some(adapter) = &self.id_lists_adapter.inner {
             match adapter
                 .clone()
                 .start(&self.statsig_runtime, self.spec_store.clone())
@@ -517,10 +536,7 @@ impl Statsig {
             log_error_to_statsig_and_console!(
                 self.ops_stats.clone(),
                 TAG,
-                StatsigErr::UnstartedAdapter(format!(
-                    "Failed to start event logging adapter: {}",
-                    e
-                ))
+                StatsigErr::UnstartedAdapter(format!("Failed to start event logging adapter: {e}"))
             );
         }
 
@@ -550,8 +566,8 @@ impl Statsig {
 
         let success = Self::start_background_tasks(
             self.statsig_runtime.clone(),
-            self.id_lists_adapter.clone(),
-            self.specs_adapter.clone(),
+            self.id_lists_adapter.inner.clone(),
+            self.specs_adapter.inner.clone(),
             self.ops_stats.clone(),
             self.background_tasks_started.clone(),
         )
@@ -706,6 +722,7 @@ impl Statsig {
 
         self.event_logger
             .enqueue(EnqueueLayerParamExpoOp::LayerOwned(
+                Utc::now().timestamp_millis() as u64,
                 Box::new(layer),
                 parameter_name,
                 ExposureTrigger::Auto,
@@ -1002,6 +1019,7 @@ impl Statsig {
         experiment.rule_id = group_id;
 
         self.event_logger.enqueue(EnqueueExperimentExpoOp {
+            exposure_time: Utc::now().timestamp_millis() as u64,
             user: &user_internal,
             experiment: &experiment,
             trigger: ExposureTrigger::Manual,
@@ -1052,7 +1070,7 @@ impl Statsig {
                 Ok(statsig)
             }
             Err(e) => {
-                let message = format!("Statsig::new_shared() mutex error: {}", e);
+                let message = format!("Statsig::new_shared() mutex error: {e}");
                 log_e!(TAG, "{}", message);
                 Err(StatsigErr::SharedInstanceFailure(message))
             }
@@ -1065,7 +1083,7 @@ impl Statsig {
                 *lock = None;
             }
             Err(e) => {
-                log_e!(TAG, "Statsig::remove_shared() mutex error: {}", e);
+                log_e!(TAG, "Statsig::remove_shared() mutex error: {e}");
             }
         }
     }
@@ -1103,6 +1121,7 @@ impl Statsig {
             self.event_logger.increment_non_exposure_checks(gate_name);
         } else {
             self.event_logger.enqueue(EnqueueGateExpoOp {
+                exposure_time: Utc::now().timestamp_millis() as u64,
                 user: &user_internal,
                 queried_gate_name: gate_name,
                 evaluation: evaluation.map(Cow::Owned),
@@ -1133,6 +1152,7 @@ impl Statsig {
             self.event_logger.increment_non_exposure_checks(gate_name);
         } else {
             self.event_logger.enqueue(EnqueueGateExpoOp {
+                exposure_time: Utc::now().timestamp_millis() as u64,
                 user: &user_internal,
                 queried_gate_name: gate_name,
                 evaluation: evaluation.as_ref().map(Cow::Borrowed),
@@ -1148,6 +1168,7 @@ impl Statsig {
         let user_internal = self.internalize_user(user);
         let (details, evaluation) = self.get_gate_evaluation(&user_internal, gate_name);
         self.event_logger.enqueue(EnqueueGateExpoOp {
+            exposure_time: Utc::now().timestamp_millis() as u64,
             user: &user_internal,
             queried_gate_name: gate_name,
             evaluation: evaluation.map(Cow::Owned),
@@ -1407,6 +1428,7 @@ impl Statsig {
                 .increment_non_exposure_checks(dynamic_config_name);
         } else {
             self.event_logger.enqueue(EnqueueConfigExpoOp {
+                exposure_time: Utc::now().timestamp_millis() as u64,
                 user: &user_internal,
                 config: &dynamic_config,
                 trigger: ExposureTrigger::Auto,
@@ -1424,6 +1446,7 @@ impl Statsig {
         let user_internal = self.internalize_user(user);
         let dynamic_config = self.get_dynamic_config_impl(&user_internal, dynamic_config_name);
         self.event_logger.enqueue(EnqueueConfigExpoOp {
+            exposure_time: Utc::now().timestamp_millis() as u64,
             user: &user_internal,
             config: &dynamic_config,
             trigger: ExposureTrigger::Manual,
@@ -1491,6 +1514,7 @@ impl Statsig {
                 .increment_non_exposure_checks(experiment_name);
         } else {
             self.event_logger.enqueue(EnqueueExperimentExpoOp {
+                exposure_time: Utc::now().timestamp_millis() as u64,
                 user: &user_internal,
                 experiment: &experiment,
                 trigger: ExposureTrigger::Auto,
@@ -1504,6 +1528,7 @@ impl Statsig {
         let user_internal = self.internalize_user(user);
         let experiment = self.get_experiment_impl(&user_internal, experiment_name);
         self.event_logger.enqueue(EnqueueExperimentExpoOp {
+            exposure_time: Utc::now().timestamp_millis() as u64,
             user: &user_internal,
             experiment: &experiment,
             trigger: ExposureTrigger::Manual,
@@ -1621,6 +1646,7 @@ impl Statsig {
 
         self.event_logger
             .enqueue(EnqueueLayerParamExpoOp::LayerOwned(
+                Utc::now().timestamp_millis() as u64,
                 Box::new(layer),
                 parameter_name,
                 ExposureTrigger::Manual,
@@ -1725,6 +1751,7 @@ impl Statsig {
             );
             return make_empty_result(EvaluationDetails::unrecognized_no_data());
         });
+
         let app_id = data.values.app_id.as_ref();
         let mut context = EvaluatorContext::new(
             user_internal,
@@ -1931,46 +1958,82 @@ fn initialize_specs_adapter(
     sdk_key: &str,
     options: &StatsigOptions,
     hashing: &HashUtil,
-) -> Arc<dyn SpecsAdapter> {
+) -> SpecsAdapterHousing {
     if let Some(adapter) = options.specs_adapter.clone() {
         log_d!(TAG, "Using provided SpecsAdapter: {}", sdk_key);
-        return adapter;
+        return SpecsAdapterHousing {
+            inner: adapter,
+            as_default_adapter: None,
+        };
     }
 
     if let Some(adapter_config) = options.spec_adapters_config.clone() {
-        return Arc::new(StatsigCustomizedSpecsAdapter::new_from_config(
+        let adapter = Arc::new(StatsigCustomizedSpecsAdapter::new_from_config(
             sdk_key,
             adapter_config,
             options,
             hashing,
         ));
+
+        return SpecsAdapterHousing {
+            inner: adapter,
+            as_default_adapter: None,
+        };
     }
 
     if let Some(data_adapter) = options.data_store.clone() {
-        return Arc::new(StatsigCustomizedSpecsAdapter::new_from_data_store(
+        let adapter = Arc::new(StatsigCustomizedSpecsAdapter::new_from_data_store(
             sdk_key,
             data_adapter,
             options,
             hashing,
         ));
+
+        return SpecsAdapterHousing {
+            inner: adapter,
+            as_default_adapter: None,
+        };
     }
 
-    Arc::new(StatsigHttpSpecsAdapter::new(sdk_key, Some(options), None))
+    let adapter = Arc::new(StatsigHttpSpecsAdapter::new(sdk_key, Some(options), None));
+
+    SpecsAdapterHousing {
+        inner: adapter.clone(),
+        as_default_adapter: Some(adapter),
+    }
 }
 
-fn initialize_id_lists_adapter(
-    sdk_key: &str,
-    options: &StatsigOptions,
-) -> Option<Arc<dyn IdListsAdapter>> {
+fn initialize_id_lists_adapter(sdk_key: &str, options: &StatsigOptions) -> IdListsAdapterHousing {
     if let Some(id_lists_adapter) = options.id_lists_adapter.clone() {
-        return Some(id_lists_adapter);
+        return IdListsAdapterHousing {
+            inner: Some(id_lists_adapter),
+            as_default_adapter: None,
+        };
     }
 
     if options.enable_id_lists.unwrap_or(false) {
-        return Some(Arc::new(StatsigHttpIdListsAdapter::new(sdk_key, options)));
+        let adapter = Arc::new(StatsigHttpIdListsAdapter::new(sdk_key, options));
+
+        return IdListsAdapterHousing {
+            inner: Some(adapter.clone()),
+            as_default_adapter: Some(adapter),
+        };
     }
 
-    None
+    IdListsAdapterHousing {
+        inner: None,
+        as_default_adapter: None,
+    }
+}
+
+struct IdListsAdapterHousing {
+    inner: Option<Arc<dyn IdListsAdapter>>,
+    as_default_adapter: Option<Arc<StatsigHttpIdListsAdapter>>,
+}
+
+struct SpecsAdapterHousing {
+    inner: Arc<dyn SpecsAdapter>,
+    as_default_adapter: Option<Arc<StatsigHttpSpecsAdapter>>,
 }
 
 fn setup_ops_stats(
